# [User Manual](@id manual)

Here we document key functionalities offered in `ClimateModels.jl`

- Climate Model Interface
- Tracked Worklow Framework
- Cloud + On-Premise File Support

## Climate Model Interface

The climate model interface is based on the [`ModelConfig`](@ref) data structure and a series of methods like [`setup`](@ref), [`build`](@ref), and [`launch`](@ref). The typical sequence is shown just below where `f` is a function that (1) receives a `ModelConfig` as its only input argument, and (2) gets called via [`launch`](@ref). 

```@example 1
using ClimateModels #hide
f=ClimateModels.RandomWalker #hide
MC=ModelConfig(model=f)
setup(MC)
build(MC)
launch(MC)
```

The `ModelConfig` called `MC` is summarized using the [`show`](@ref) method which here reveals that `f` is just an alias for `ClimateModels.RandomWalker`. The run folder name can be accessed directly using [`pathof`](@ref).


```@example 1
MC #hide
```

The content of the run folder is readily inspected using [`readdir`]().

```@example 1
readdir(pathof(MC)) #hide
```

Here `launch(MC)` has completed, `RandomWalker.csv` is a file that was generated by function `f`, and `log` is the log subfolder that was created by [`setup`](@ref). 

The workflow log is retrieved using the [`log`](@ref) function. 

```@example 1
log(MC) #hide
```

This highlights that `Project.toml` and `Manifest.toml` for the environment being used have been archived. This happens during [`setup`](@ref) to document all dependencies and make the workflow reproducible.

### Generalization

A key point is that everything can be customized to, e.g., use popular models previously written in Fortran or C just as simply. This typically involves defining a new concrete type of `AbstractModelConfig` and providing a customized `build` method to compile the model. 

This approach is illustrated in the [examples](@ref) as well as in the suite of general circulation model examples that uses the customized interface elements provided by [MITgcmTools.jl](https://github.com/gaelforget/MITgcmTools.jl) for [MITgcm](https://mitgcm.readthedocs.io/en/latest/).

The idea in the longer term is that for popular models the customized interface elements would be provided via a dedicated package (e.g. [MITgcmTools.jl](https://github.com/gaelforget/MITgcmTools.jl)). These customized interfaces would thus be maintained independently by developers and users most familiar with each model.

## Git / Log Support

The `setup` method normally calls [`log`](@ref) to set up a temporary run folder with a `git` enabled subfolder called `log`. This allows for recording each workflow step, using `log` functions listed below.

```@docs
log
```

## Cloud / File Support

There are various ways that model output gets archived, distributed, and retrieved from the internet (e.g., see [`cmip`](@ref)). In some cases downloading data can be the most convenient approach. In others it can be more advantageous to compute in the cloud and only download final results for plotting. 

`ClimateModels.jl` comes equiped with packages that read popular file formats used in climate modeling and science. [Downloads.jl](https://github.com/JuliaLang/Downloads.jl), [CSV.jl](https://github.com/JuliaData/CSV.jl), [DataFrames.jl](https://github.com/JuliaData/DataFrames.jl), [NetCDF.jl](https://github.com/JuliaGeo/NetCDF.jl), [Zarr.jl](https://github.com/meggart/Zarr.jl), and [TOML.jl](https://github.com/JuliaLang/TOML.jl) are thus readily available when you install `ClimateModels.jl`. 

For example, one can read the CSV file generated by `ClimateModels.RandomWalker` as follows:

```@example 1
fil=joinpath(pathof(MC),"RandomWalker.csv")
ClimateModels.CSV.File(fil) |> ClimateModels.DataFrame
show(ans) #hide
```

## API Reference

```@index
```

```@docs
ModelConfig
```

```@docs
setup
build
launch
show
pathof
readdir
clean
```

```@docs
cmip
```
